# TWO queues, two precedences: AF-policy

# Set DS RED parameters for HTTP:
#$qE1C meanPktSize $packetSize

 # Committed transfer rates for applications (per connection pair)
set cir_exp   80000
set cir_http 100000
set cir_ftp  200000

# Assign initial DS-codepoints
set EF(cp) 10
set AF(cp) 20
set BE(cp) 30

# Assign queue weights for WRR-scheduler
set EF(qw) 60
set AF(qw) 20
set BE(qw) 20

# Set RED parameters:
 # BE
set BE(in_min) 25
set BE(in_max) 80
set BE(in_prob) 0.05
set BE(out_min) 25
set BE(out_max) 80
set BE(out_prob) 0.05
 # AF
set AF(in_min) 45
set AF(in_max) 80
set AF(in_prob) 0.05
set AF(out_min) 25
set AF(out_max) 80
set AF(out_prob) 0.10
 # EF
set EF(in_min) 45
set EF(in_max) 80
set EF(in_prob) 0.05
set EF(out_min) 80
set EF(out_max) 80
set EF(out_prob) 1.00

# DiffServ Core queue (one-way: from a to b)
proc confDSCore { a b } {
    global ns q n AF EF BE

    set q($a$b) [[$ns link $n($a) $n($b)] queue]

    # Set scheduler (and queue weigths)
    $q($a$b) setSchedularMode WRR      ;# Options: RR, WRR, WIRR, PRI
    $q($a$b) addQueueWeights 0 $EF(qw) ;# Syntax: $q addQueueWeights [queue] [weight]
    $q($a$b) addQueueWeights 1 $AF(qw)
    $q($a$b) addQueueWeights 2 $BE(qw)

    $q($a$b) set numQueues_ 3
    $q($a$b) setNumPrec 2
    $q($a$b) meanPktSize 300
    $q($a$b) set limit_ 100

    $q($a$b) addPHBEntry $EF(cp) 0 0 ;# EF
    $q($a$b) addPHBEntry [expr $EF(cp)+1] 0 1
    $q($a$b) configQ 0 0 $EF(in_min) $EF(in_max) $EF(in_prob)
    $q($a$b) configQ 0 1 $EF(out_min) $EF(out_max) $EF(out_prob)
    $q($a$b) setPhysQueueSize 0 10
    
    $q($a$b) addPHBEntry $AF(cp) 1 0 ;# AF
    $q($a$b) addPHBEntry [expr $AF(cp)+1] 1 1
    $q($a$b) configQ 1 0 $AF(in_min) $AF(in_max) $AF(in_prob)
    $q($a$b) configQ 1 1 $AF(out_min) $AF(out_max) $AF(out_prob)
    
    $q($a$b) addPHBEntry $BE(cp) 2 0 ;# BE
    $q($a$b) addPHBEntry [expr $BE(cp)+1] 2 1
    $q($a$b) configQ 2 0 $BE(in_min) $BE(in_max) $BE(in_prob)
    $q($a$b) configQ 2 1 $BE(out_min) $BE(out_max) $BE(out_prob)
}

# Configure DS edge queues for access links, both server- and client-side
proc confDSEdges { svr clnt cir app service_type } {
    global ns q n cir_exp cir_http cir_ftp BE EF AF

    if { $service_type == "BE" } {
	set cp_ $BE(cp)
	set in_min_ $BE(in_min)
	set in_max_ $BE(in_max)
	set in_prob_ $BE(in_prob)
	set out_min_ $BE(in_min)
	set out_max_ $BE(in_max)
	set out_prob_ $BE(in_prob)
    } elseif { $service_type == "AF" } {
	set cp_ $AF(cp)
	set in_min_ $AF(in_min)
	set in_max_ $AF(in_max)
	set in_prob_ $AF(in_prob)
	set out_min_ $AF(in_min)
	set out_max_ $AF(in_max)
	set out_prob_ $AF(in_prob)
    } elseif { $service_type == "EF" } {
	set cp_ $EF(cp)
	set in_min_ $EF(in_min)
	set in_max_ $EF(in_max)
	set in_prob_ $EF(in_prob)
	set out_min_ $EF(in_min)
	set out_max_ $EF(in_max)
	set out_prob_ $EF(in_prob)
    }

    # Edge queue
    set q($svr$clnt) [[$ns link $n($svr) [$n($svr) neighbors]] queue]
    set q($clnt$svr) [[$ns link $n($clnt) [$n($clnt) neighbors]] queue]

    foreach qe "$svr$clnt $clnt$svr" {
	$q($qe) set numQueues_ 1
	$q($qe) setNumPrec 2
	$q($qe) meanPktSize 300
	$q($qe) set limit_ 100
	
	$q($qe) addPolicyEntry -1 -1 TSW2CM $cp_ $cir $app
	if { $service_type == "BE" } {
	    $q($qe) addPolicerEntry TSW2CM $cp_ $cp_
	} else {
	    $q($qe) addPolicerEntry TSW2CM $cp_ [expr $cp_ + 1]
	}
	$q($qe) addPHBEntry $cp_ 0 0
	$q($qe) addPHBEntry [expr $cp_ + 1] 0 1
	$q($qe) configQ 0 0 $in_min_ $in_max_ $in_prob_
	$q($qe) configQ 0 1 $out_min_ $out_max_ $out_prob_
    }
}
