set ns [new Simulator]

 # Simulated seconds and random seed to be used (0 := random seed for every run)
set testTime 180.0
$defaultRNG seed 1

 # Activate tracing for all links (potentially very slow and disk-space-consuming)
 # Output file: tr_diffnet.out
set trace_all false

 # Tracing for the bottleneck link. Output file: tr_bottleneck.out
set trace_bn false

 # Set monitoring for queue length (some performance cost - adjust 'q_interval' below)
 # Outputfiles: q_r6r7.mon, q_r6r7_avg.mon, q_r7r6.mon, q_r7r6_avg.mon.
set monitor_qlengths true



Trace set show_tcphdr_ 1
 # Set tracing
if {$trace_all} {
    $ns trace-all [open tr_diffnet.out w]
    $ns namtrace-all [open tr_diffnet.nam w]
}

 # Get functions for creating HTTP, FTP and VoIP traffic
source peer_setup.tcl

 # Load DiffServ-specific procedures
source 2q2p.tcl

 # Select network type (BE, RED, ECN, EF, AF)
source conf/netType.tcl

 # Create topology
source topology.tcl

 # Load monitor-functions for queue- and flowmonitors
source monitoring.tcl

if {$trace_bn} {
    set tf [open "tr_bottleneck.out" w]
    $ns trace-queue $n(r6) $n(r7) $tf
    $ns trace-queue $n(r7) $n(r6) $tf
}

 # Print node's name and ID on the screen
foreach s [set node_list] {
    puts -nonewline "Node $s's id is [$n($s) address?] and its neighbors are"
    foreach nbor [$n($s) neighbors] {
	puts -nonewline " [$nbor address?]"
    }
    puts ""
}

 # Contains handles for WebTraf- and FTPTraf-sessions. Session handles are added
 # to this variable in "proc launchSession" of file 'peer_setup.tcl'
$ns set sessionList ""

 # Fix some of sources / destinations
source conf/sources.tcl
 # Launch HTTP-sessions (syntax: client-list server-list)
launchHttp httpC1 httpS4
launchHttp httpC2 httpS3
launchHttp httpC3 httpS1
launchHttp httpC4 httpS2

 # Launch FTP-sessions (syntax: client-list server-list)
launchFtp ftpC1 ftpS4
launchFtp ftpC2 ftpS3
launchFtp ftpC3 ftpS2
launchFtp ftpC4 ftpS1

 # Launch VOIP-sessions (syntax: src dst id)
launchVoip voip1 voip5 0
launchVoip voip5 voip1 1
launchVoip voip2 voip7 2
launchVoip voip7 voip2 3
launchVoip voip3 voip6 4
launchVoip voip6 voip3 5
launchVoip voip4 voip8 6
launchVoip voip8 voip4 7


 # Queue-monitoring (for bottleneck link)
set qf [open "qmon_bottleneck.mon" w]
set qmon [$ns monitor-queue $n(r6) $n(r7) $qf]
set qmon2 [$ns monitor-queue $n(r7) $n(r6) $qf]
$ns at 1.0 "record $qmon $n(r6) $n(r7) 1 $qf"
$ns at 1.0 "record $qmon2 $n(r7) $n(r6) 1 $qf"

 # Finish up the simulation
proc finish {} {
    global ns alku trace_on pf
    $ns flush-trace
    puts "Ajoaika: [expr [clock seconds] - $alku] s"

    flush $pf
    close $pf
    #xec xgraph performance.mon

    exit 0
}

 # Tries to close all open connections listed in 'sessionList' using the normal TCP-connection teardown
proc close-connections {} {
    global ns
    foreach s [$ns set sessionList] {
	$s close-pool
    }
}

 # Forces all connections to be closed, no matter what the state of the connection is.
 # Needed to ensure statistics to be collected after all connections are closed.
proc kill-em-all {} {
    global ns
    foreach s [$ns set sessionList] {
	$s kill-kill-kill
    }
}


 # Some performance statistics (in Xgraph-compliant format)
set pf [open performance.mon w]
puts $pf "0 0"

 # Some diffserv-specific monitoring stuff
#diffservStats

set alku [clock seconds]
$ns at 100 "timeStats 100"

$ns at $testTime "close-connections"      ;# Try to cleanly close all connections (by sending 'fin')
$ns at [expr $testTime + 5] "kill-em-all" ;# Force all remaining connections to be closed
$ns at [expr $testTime + 11] "finish"     ;# End the simulation
$ns run