set ns [new Simulator]

set testTime 50
$defaultRNG seed 1

# variables to control tracing
# All links & NAM (slow, lots of disk space)
set trace_all false
# Tracing for the bottleneck link. Output file: tr_bottleneck.out
set trace_bn false
# Monitor queue lengths
set monitor_qlengths true

if {$trace_all} {
    $ns trace-all [open tr_diffnet.out w]
    $ns namtrace-all [open tr_diffnet.nam w]
}

# Select network type (BE, RED, ECN, EF, AF)
source conf/netType.tcl


# Create topology
source topology.tcl


# Set up one CBR connection between each source and the destination:
set udp1 [new Agent/UDP]
$ns attach-agent $s1 $udp1
set cbr1 [new Application/Traffic/CBR]
$cbr1 attach-agent $udp1
$cbr1 set packet_size_ $packetSize
$udp1 set packetSize_ $packetSize
$cbr1 set rate_ $rate1
set null1 [new Agent/Null]
$ns attach-agent $dest $null1
$ns connect $udp1 $null1

set udp2 [new Agent/UDP]
$ns attach-agent $s2 $udp2
set cbr2 [new Application/Traffic/CBR]
$cbr2 attach-agent $udp2
$cbr2 set packet_size_ $packetSize
$udp2 set packetSize_ $packetSize
$cbr2 set rate_ $rate2
set null2 [new Agent/Null]
$ns attach-agent $dest $null2
$ns connect $udp2 $null2

set tcp3 [new Agent/TCP]
$ns attach-agent $s3 $tcp3
set ftp3 [new Application/FTP]
$ftp3 attach-agent $tcp3
$tcp3 set packetSize_ $packetSize
set sink3 [new Agent/TCPSink]
$ns attach-agent $dest $sink3
$ns connect $tcp3 $sink3


set tcp4 [new Agent/TCP]
$ns attach-agent $s4 $tcp4
set ftp4 [new Application/FTP]
$ftp4 attach-agent $tcp4
$tcp4 set packetSize_ $packetSize
set sink4 [new Agent/TCPSink]
$ns attach-agent $dest $sink4
$ns connect $tcp4 $sink4


set tcp5 [new Agent/TCP]
$ns attach-agent $s5 $tcp5
set ftp5 [new Application/FTP]
$ftp5 attach-agent $tcp5
$tcp5 set packetSize_ $packetSize
set sink5 [new Agent/TCPSink]
$ns attach-agent $dest $sink5
$ns connect $tcp5 $sink5


set tcp6 [new Agent/TCP]
$ns attach-agent $s6 $tcp6
set ftp6 [new Application/FTP]
$ftp6 attach-agent $tcp6
$tcp6 set packetSize_ $packetSize
set sink6 [new Agent/TCPSink]
$ns attach-agent $dest $sink6
$ns connect $tcp6 $sink6


set tcp7 [new Agent/TCP]
$ns attach-agent $s7 $tcp7
set ftp7 [new Application/FTP]
$ftp7 attach-agent $tcp7
$tcp7 set packetSize_ $packetSize
set sink7 [new Agent/TCPSink]
$ns attach-agent $dest $sink7
$ns connect $tcp7 $sink7


set tcp8 [new Agent/TCP]
$ns attach-agent $s8 $tcp8
set ftp8 [new Application/FTP]
$ftp8 attach-agent $tcp8
$tcp8 set packetSize_ $packetSize
set sink8 [new Agent/TCPSink]
$ns attach-agent $dest $sink8
$ns connect $tcp8 $sink8


set tcp9 [new Agent/TCP]
$ns attach-agent $s9 $tcp9
set ftp9 [new Application/FTP]
$ftp9 attach-agent $tcp9
$tcp9 set packetSize_ $packetSize
set sink9 [new Agent/TCPSink]
$ns attach-agent $dest $sink9
$ns connect $tcp9 $sink9


set tcp10 [new Agent/TCP]
$ns attach-agent $s10 $tcp10
set ftp10 [new Application/FTP]
$ftp10 attach-agent $tcp10
$tcp10 set packetSize_ $packetSize
set sink10 [new Agent/TCPSink]
$ns attach-agent $dest $sink10
$ns connect $tcp10 $sink10



proc finish {} {
    global ns trace_all
    close $trace_all

    #Following are awk scripts to process trace of all events.

    exec awk {
	{
	    if (($1 == "r") && ($4 == "12"))\
		    print $2, $9, $6
	}
    } out.all > out.rec
	

    exit 0
}


$qE1E2 printPolicyTable
$qE1E2 printPolicerTable

$ns at 0.0 "$cbr1 start"
$ns at 0.0 "$cbr2 start"
$ns at 0.0 "$ftp3 start"
$ns at 0.0 "$ftp4 start"
$ns at 0.0 "$ftp5 start"
$ns at 0.0 "$ftp6 start"
$ns at 0.0 "$ftp7 start"
$ns at 0.0 "$ftp8 start"
$ns at 0.0 "$ftp9 start"
$ns at 0.0 "$ftp10 start"
$ns at 10.0 "$qE1E2 printStats"
$ns at 20.0 "$qE1E2 printStats"
$ns at 30.0 "$qE1E2 printStats"
$ns at 40.0 "$qE1E2 printStats"
$ns at $testTime "$cbr1 stop"
$ns at $testTime "$cbr2 stop"
$ns at $testTime "$ftp3 stop"
$ns at $testTime "$ftp4 stop"
$ns at $testTime "$ftp5 stop"
$ns at $testTime "$ftp6 stop"
$ns at $testTime "$ftp7 stop"
$ns at $testTime "$ftp8 stop"
$ns at $testTime "$ftp9 stop"
$ns at $testTime "$ftp10 stop"
$ns at [expr $testTime + 1.0] "finish"

$ns run
