# Outputs to the screen the simulated time and the test time every 'interval'-seconds (simulated)
proc timeStats { interval } {
    global ns alku pf

    set openConnections 0
    foreach s [$ns set sessionList] {
	set temp [llength [$s set connList]]
	puts "[$s info class] open connections: $temp"
	set openConnections [expr $openConnections + $temp]
    }
    puts "Total: $openConnections\n"

    # Print simulated and runtime on the screen and to file
    puts "\nSimulated time: [$ns now] s, real time: [expr [clock seconds] - $alku] s"
    puts $pf "[$ns now] [expr [clock seconds] - $alku]   Open connections $openConnections Number of events [$ns numEvents]"

    $ns at [expr [$ns now] + $interval] "timeStats $interval" ;# Call timeStats in 100s intervals
}

# Print sample and average queue lengths to files every 'interval' seconds (simulated)
proc printDSQueueLength { q q_num interval outfile outfile_avg } {
    global ns
    set now_ [format %.2f [$ns now]]
    foreach qn $q_num {
	puts $outfile "$now_ queue number $qn: length [$q getCurrent $qn]"
	puts $outfile_avg "$now_ queue number $qn: length [$q getAverage $qn]"
    }
    $ns at [expr [$ns now]+$interval] "printDSQueueLength $q [list $q_num] $interval $outfile $outfile_avg"
}

 # DiffServ-specific statistics
proc diffservStats {} {
    global ns q testTime monitor_qlengths
    
    # Print policy and policer information before simulation starts
    #$ns at 0.0 "$q(r6r7) getStat"
    $ns at 0.0 "$q(r6r7) printPHBTable"
    $ns at 0.0 "$q(voip1voip5) printPolicyTable"
    $ns at 0.0 "$q(ftpS1ftpC4) printPolicyTable"
    $ns at 0.0 "$q(httpS1httpC3) printPolicyTable"

    # Print sample and avarage queue size for the bottleneck link every 'q_interval' seconds
    if { $monitor_qlengths } {
	set q_interval 0.2                      ;# Sample interval
	set q_num [list "0 1 2"]                ;# Physical queues in diffserv routers
	set q_r6r7 [open q_r6r7.mon w]          ;# Output files for sample qlengths
	set q_r7r6 [open q_r7r6.mon w]          ;#
	set q_r6r7_avg [open q_r6r7_avg.mon w]  ;# Output files for average qlengths
	set q_r7r6_avg [open q_r7r6_avg.mon w]  ;#
	$ns at $q_interval "printDSQueueLength $q(r6r7) $q_num $q_interval $q_r6r7 $q_r6r7_avg"
	$ns at $q_interval "printDSQueueLength $q(r7r6) $q_num $q_interval $q_r7r6 $q_r7r6_avg"
    }

    # Print statistics for diffserv bottleneck routers every 'interval' seconds and at the end
    set interval 50
    for {set i 1} {$i < [expr $testTime/$interval]} {incr i} {
	$ns at [expr $interval*$i] "$q(r6r7) printStats"
	$ns at [expr $interval*$i] "$q(r7r6) printStats"
    }
    $ns at $testTime "$q(r6r7) printStats"
    $ns at $testTime "$q(r7r6) printStats"
}

# Dumps queue-statistics in server- and client-side access-links every 'interval' seconds (simulated).
proc crQmon { svr clnt interval } {
    global ns n
    set qf [open "qmon_[set svr]_to_[set clnt].mon" w]
    set qmon [$ns monitor-queue $n($svr) [$n($svr) neighbors] $qf]
    set qmon2 [$ns monitor-queue [$n($clnt) neighbors] $n($clnt) $qf]
    #$qmon trace $qf

    record $qmon $n($svr) [$n($svr) neighbors] $interval $qf
    record $qmon2 [$n($clnt) neighbors] $n($clnt) $interval $qf
}

proc record { qmon src dst interval file} {
    global ns

    set time  [$ns now]
    set size [$qmon set size_]
    set pkts [$qmon set pkts_]
    set parrivals [$qmon set parrivals_]
    set barrivals [$qmon set barrivals_]
    #set pdepartures [$qmon set pdepartures_]
    #set bdepartures [$qmon set bdepartures_]
    set pdrops [$qmon set pdrops_]
    set bdrops [$qmon set bdrops_]

    #puts $file "$time size=$size, pkts=$pkts, parrivals=$parrivals, barrivals=$barrivals, pdepartures=$pdepartures, bdepartures=$bdepartures, pdrops=$pdrops, bdrops=$bdrops"
    puts $file "$time [$src id]->[$dst id] size $size  pkts $pkts parrivals $parrivals barrivals $barrivals pdrops $pdrops bdrops $bdrops"
    flush $file
    #$qmon reset

    # stats every 1 second
    $ns at [expr $time+$interval] "record $qmon $src $dst $interval $file"
}

# Dumps statistics of flows in 'fmon' every 'interval' seconds (simulated)
proc flowdump { fmon fm interval } {
    global ns testTime
    $fmon dump
    flush $fm

    set aika [$ns now]
    foreach f [$fmon flows] {

        set flow_id [$f set flowid_]
        set flow_p_arrs [$f set parrivals_]
        set flow_p_deps [$f set pdepartures_]
        set flow_p_drops [$f set pdrops_]
        set flow_arrs [$f set barrivals_]
        set flow_tp [expr round($flow_arrs * 8 / $testTime)]
        set flow_deps [$f set bdepartures_]
        set flow_drops [$f set bdrops_]
        #set bthroughput [expr round($flow_p_deps * 8 * 576 / $testTime)]
        #puts "time=$aika, id=$flow_id, parrivals=$flow_p_arrs, pdrops=$flow_p_drops, barrivals=$flow_arrs, bdepartures=$flow_deps, bdrops=$flow_drops"
        #puts "time=$aika, id=$flow_id, parrivals=$flow_p_arrs, pdepartures=$flow_p_deps, pdrops=$flow_p_drops, fmonTP=$flow_tp"
        $f reset
    }
    $fmon reset
    $ns at [expr $aika + $interval] "flowdump $fmon $fm $interval"
}
